#!/usr/bin/python
import json
import sys

from charmhelpers.core.hookenv import (
    config,
    log,
    Hooks,
    relation_get,
    relation_ids,
    relation_set,
    UnregisteredHookError,
    ERROR,
)

from charmhelpers.fetch import apt_install

hooks = Hooks()

NET_MANAGER_PACKAGES = {
    'flatdhcpmanager': ['nova-network'],
}


@hooks.hook()
def install():
    # install source should have been configured by primary
    # currently install nova-network. Currently we need to
    # install at least nova-compute-qemu to get the service
    # setup correctly.
    apt_install(['nova-compute-qemu'], fatal=True)


def install_net_manager_packages(net_manager):
    try:
        pkgs = NET_MANAGER_PACKAGES[net_manager]
    except KeyError:
        log('Principle specified unsupported network manager: %s.' %
            net_manager)
        raise

    log('Installing %s to support %s network manager.' % (pkgs, net_manager))
    apt_install(pkgs, fatal=True)


def prepare_relation_context():
    '''converts current charm config to subordinate config interface'''
    conf = config()
    vmw_config = {
        'compute_driver': 'vmwareapi.VMwareVCDriver',
        'flat_network_bridge': conf['flat_network_bridge'],
        'vmwareapi_host_ip': conf['vmwareapi_ip'],
        'vmwareapi_host_username': conf['vmwareapi_username'],
        'vmwareapi_host_password': conf['vmwareapi_password'],
        'vmwareapi_cluster_name': conf['vmwareapi_cluster_name'],
        'vmwareapi_vlan_interface': conf['vmwareapi_vlan_interface'],
        'vmwareapi_integration_bridge': conf['vmwareapi_integration_bridge'],
    }

    # XXX: Had to update this for new round of testing.  We can potentially
    #      also setup a local tomcat6 server to server this and figure out
    #      a way of fetching or distributing the SDK with the charm.
    wsdl_loc = 'http://%s/sdk/vimService.wsdl' % conf['vmwareapi_ip']
    vmw_config['vmwareapi_wsdl_loc'] = wsdl_loc

    missing = [k for k, v in vmw_config.iteritems() if not v or v is '']
    if missing:
        log('Missing required config settings: %s' % missing, level=ERROR)
        raise

    # as specified in http://pad.ubuntu.com/redux-testing (10/08/13)
    rel_config = {
        'nova': {
            '/etc/nova/nova.conf': {
                'sections': {
                    'DEFAULT': [[k, v] for k, v in vmw_config.iteritems()]
                }
            }
        }
    }

    return rel_config


@hooks.hook()
def nova_vmware_relation_changed():
    net_manager = relation_get('network_manager')

    if not net_manager:
        log('Network manager not yet set by principle.')
        return

    install_net_manager_packages(net_manager)
    config_changed()


@hooks.hook()
def config_changed():
    settings = prepare_relation_context()
    # forward over all relevant config for principles to add to their config
    for rid in relation_ids('nova-vmware'):
        relation_set(subordinate_configuration=json.dumps(settings),
                     relation_id=rid)


if __name__ == '__main__':
    try:
        hooks.execute(sys.argv)
    except UnregisteredHookError as e:
        log('Unknown hook {} - skiping.'.format(e))
